var Container = typeof Buffer !== 'undefined'
  ? Buffer // in node, use buffers
  : Uint8Array // in browsers, use typed arrays

function BitField (data, opts) {
  if (!(this instanceof BitField)) {
    return new BitField(data, opts)
  }

  if (arguments.length === 0) {
    data = 0
  }

  this.grow = (opts && ((isFinite(opts.grow) && getByteSize(opts.grow)) || opts.grow)) || 0

  if (typeof data === 'number' || data === undefined) {
    data = new Container(getByteSize(data)) // eslint-disable-line node/no-deprecated-api
    if (data.fill && !data._isBuffer) data.fill(0) // clear node buffers of garbage
  }
  this.buffer = data
}

function getByteSize (num) {
  var out = num >> 3
  if (num % 8 !== 0) out++
  return out
}

BitField.prototype.get = function (i) {
  var j = i >> 3
  return (j < this.buffer.length) &&
    !!(this.buffer[j] & (128 >> (i % 8)))
}

BitField.prototype.set = function (i, b) {
  var j = i >> 3
  if (b || arguments.length === 1) {
    if (this.buffer.length < j + 1) {
      this._grow(Math.max(j + 1, Math.min(2 * this.buffer.length, this.grow)))
    }
    // Set
    this.buffer[j] |= 128 >> (i % 8)
  } else if (j < this.buffer.length) {
    // Clear
    this.buffer[j] &= ~(128 >> (i % 8))
  }
}

BitField.prototype._grow = function (length) {
  if (this.buffer.length < length && length <= this.grow) {
    var newBuffer = new Container(length) // eslint-disable-line node/no-deprecated-api
    if (newBuffer.fill) newBuffer.fill(0)
    if (this.buffer.copy) {
      this.buffer.copy(newBuffer, 0)
    } else {
      for (var i = 0; i < this.buffer.length; i++) {
        newBuffer[i] = this.buffer[i]
      }
    }
    this.buffer = newBuffer
  }
}

if (typeof module !== 'undefined') module.exports = BitField
